/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.tuple.api.Tuple;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
// Import clauses required by the object Olstore_item
import jorm.Olstore_itemAccessor;
// Imports needed by the rdb mapper
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.io.IOException;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;

public class Olstore_itemBinding
	extends org.objectweb.jonas_ejb.container.JEntitySwitchCS
	implements PBindingCtrl
      {
	/**
	 * The current state of this PBinding object.
	 */
	transient protected byte status;
	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	transient public Olstore_itemMapping pClassMapping;
	/**
	 * The PName of this PBinding object.
	 */
	transient protected PName pname;
	transient RdbAdapter adapter;

	public Olstore_itemBinding() {
// nothing
	}

	public Olstore_itemBinding(PClassMapping pcm) {
		pClassMapping = (Olstore_itemMapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (Olstore_itemMapping) pcm;
		status = PBinding.LIFECYCLE_NOTBOUND;
	    pname = null;
		if (pcm != null) {
			PMapperRdb mapper = (PMapperRdb) pcm.getPMapper();
			if (mapper != null) {
				adapter = mapper.getRdbAdapter();
			}
		}
		if (adapter == null) {
			throw new PException("No mapper found throught the PClassMapping: " + pcm);
		}
	}

	public void write(Object conn, PAccessor pa)
		throws PException {
		Olstore_itemAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Olstore_itemAccessor)
			paccessor = (Olstore_itemAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		    if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
//Table olstore_item : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deleteolstore_itemTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_item]: problematic SQL query -> {" + sqlquery + "}");
                }
            } else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertolstore_itemTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_item]: problematic SQL query -> {" + sqlquery + "}");
                }
		    } else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updateolstore_itemTable(connection, conn, paccessorGen, paccessor);
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [Olstore_item]: problematic SQL query -> {" + sqlquery + "}");
                }
		    }
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Read");
		Olstore_itemAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Olstore_itemAccessor)
			paccessor = (Olstore_itemAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            pstmt = connection.prepareStatement(pClassMapping.readQuery);
            adapter.setOint(pstmt, 1, pname.encodeOint());
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false)
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " + pname);
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetItemId(adapter.getOint(resultSet, 1, null));
                paccessor.paSetLongDesc(adapter.getString(resultSet, 2, null));
                paccessor.paSetMainPage(adapter.getOboolean(resultSet, 3, null));
                paccessor.paSetName(adapter.getString(resultSet, 4, null));
                paccessor.paSetPrice(adapter.getBigDecimal(resultSet, 6, null));
                paccessor.paSetShortDesc(adapter.getString(resultSet, 7, null));
                paccessor.paSetSpecialOffer(adapter.getOboolean(resultSet, 8, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1019275
				//fieldName=typeId_name
                fieldval = adapter.getString(resultSet, 5, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.typeIdPnc.decodeString(null);
                } else {
                    refname = pClassMapping.typeIdPnc.decodeString((java.lang.String) fieldval);
                }
                paccessor.paSetTypeId(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@e6c6d7
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.jonasCMRUser_ItemPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.jonasCMRUser_ItemPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetJonasCMRUser_Item(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@17ccb2f
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.jonasCMROrder_ItemPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.jonasCMROrder_ItemPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetJonasCMROrder_Item(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1ff8de3
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.propertiesPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.propertiesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetProperties(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bc448b
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.picturesPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessor.paSetPictures(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetOintField(pClassMapping.itemId_NAME, adapter.getOint(resultSet, 1, null));
                paccessorGen.paSetStringField(pClassMapping.longDesc_NAME, adapter.getString(resultSet, 2, null));
                paccessorGen.paSetObooleanField(pClassMapping.mainPage_NAME, adapter.getOboolean(resultSet, 3, null));
                paccessorGen.paSetStringField(pClassMapping.name_NAME, adapter.getString(resultSet, 4, null));
                paccessorGen.paSetBigDecimalField(pClassMapping.price_NAME, adapter.getBigDecimal(resultSet, 6, null));
                paccessorGen.paSetStringField(pClassMapping.shortDesc_NAME, adapter.getString(resultSet, 7, null));
                paccessorGen.paSetObooleanField(pClassMapping.specialOffer_NAME, adapter.getOboolean(resultSet, 8, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bc448b
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.picturesPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(pClassMapping.pictures_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bc448b
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.picturesPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(pClassMapping.pictures_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bc448b
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.picturesPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(pClassMapping.pictures_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bc448b
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.picturesPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(pClassMapping.pictures_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bc448b
				//fieldName=itemId
                fieldval = adapter.getOint(resultSet, 1, null);
                if (resultSet.wasNull()) {
                    refname = pClassMapping.picturesPnc.decodeOint(null);
                } else {
                    refname = pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fieldval);
                }
                paccessorGen.paSetRefField(pClassMapping.pictures_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		status = nextState;
	}

	public void read(Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = pClassMapping.getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(conn, pa);
	        return;
	    }
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, pClassMapping, txctx);
	        if (tuple == null) {
	            read(conn, pa);
	            return;
	        }
		    Olstore_itemAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof Olstore_itemAccessor)
			    paccessor = (Olstore_itemAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
                paccessor.paSetItemId((Integer) tuple.getObject(1));
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessor.paSetJonasCMROrder_Item(pClassMapping.jonasCMROrder_ItemPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetJonasCMROrder_Item(pClassMapping.jonasCMROrder_ItemPnc.getNull(), conn);
                }
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessor.paSetJonasCMRUser_Item(pClassMapping.jonasCMRUser_ItemPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetJonasCMRUser_Item(pClassMapping.jonasCMRUser_ItemPnc.getNull(), conn);
                }
                paccessor.paSetLongDesc(tuple.getString(2));
                paccessor.paSetMainPage((Boolean) tuple.getObject(3));
                paccessor.paSetName(tuple.getString(4));
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessor.paSetPictures(pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetPictures(pClassMapping.picturesPnc.getNull(), conn);
                }
                paccessor.paSetPrice((BigDecimal) tuple.getObject(6));
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessor.paSetProperties(pClassMapping.propertiesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessor.paSetProperties(pClassMapping.propertiesPnc.getNull(), conn);
                }
                paccessor.paSetShortDesc(tuple.getString(7));
                paccessor.paSetSpecialOffer((Boolean) tuple.getObject(8));
				fval =
					tuple.getString(5);
                if (tuple.isDefined(5)) {
                    paccessor.paSetTypeId(pClassMapping.typeIdPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessor.paSetTypeId(pClassMapping.typeIdPnc.getNull(), conn);
                }
            } else if (paccessorGen != null) {
                paccessorGen.paSetOintField(pClassMapping.itemId_NAME, (Integer) tuple.getObject(1));
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(pClassMapping.jonasCMROrder_Item_NAME, pClassMapping.jonasCMROrder_ItemPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(pClassMapping.jonasCMROrder_Item_NAME, pClassMapping.jonasCMROrder_ItemPnc.getNull(), conn);
                }
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(pClassMapping.jonasCMRUser_Item_NAME, pClassMapping.jonasCMRUser_ItemPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(pClassMapping.jonasCMRUser_Item_NAME, pClassMapping.jonasCMRUser_ItemPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(pClassMapping.longDesc_NAME, tuple.getString(2));
                paccessorGen.paSetObooleanField(pClassMapping.mainPage_NAME, (Boolean) tuple.getObject(3));
                paccessorGen.paSetStringField(pClassMapping.name_NAME, tuple.getString(4));
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(pClassMapping.pictures_NAME, pClassMapping.picturesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(pClassMapping.pictures_NAME, pClassMapping.picturesPnc.getNull(), conn);
                }
                paccessorGen.paSetBigDecimalField(pClassMapping.price_NAME, (BigDecimal) tuple.getObject(6));
				fval =
					(Integer) tuple.getObject(1);
                if (tuple.isDefined(1)) {
                    paccessorGen.paSetRefField(pClassMapping.properties_NAME, pClassMapping.propertiesPnc.decodeOint((java.lang.Integer) fval), conn);
                } else {
                    paccessorGen.paSetRefField(pClassMapping.properties_NAME, pClassMapping.propertiesPnc.getNull(), conn);
                }
                paccessorGen.paSetStringField(pClassMapping.shortDesc_NAME, tuple.getString(7));
                paccessorGen.paSetObooleanField(pClassMapping.specialOffer_NAME, (Boolean) tuple.getObject(8));
				fval =
					tuple.getString(5);
                if (tuple.isDefined(5)) {
                    paccessorGen.paSetRefField(pClassMapping.typeId_NAME, pClassMapping.typeIdPnc.decodeString((java.lang.String) fval), conn);
                } else {
                    paccessorGen.paSetRefField(pClassMapping.typeId_NAME, pClassMapping.typeIdPnc.getNull(), conn);
                }
            }
		    status = nextState;
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Exist");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		status = nextState;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT olstore_item.itemId_ FROM olstore_item WHERE olstore_item.itemId_ = ?");
            adapter.setOint(pstmt, 1, pname.encodeOint());
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}

	public PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Export with hints");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unexport");
        pClassMapping.getPBinder().unexport(conn, this.pname, this);
	}

	public void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Bind");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Unbind");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}

    private String updateolstore_itemTable(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        Olstore_itemAccessor paccessor)
        throws PException, SQLException, IOException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
            pstmt = connection.prepareStatement(res = "UPDATE olstore_item SET name_ = ?, specialOffer_ = ?, price_ = ?, itemId_ = ?, longDesc_ = ?, shortDesc_ = ?, mainPage_ = ?, olstore_type_name_ = ? WHERE itemId_ = ?");
				//name_
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//specialOffer_
                nullField = paccessor.paGetSpecialOffer();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 2, ((java.lang.Boolean) nullField));
                }
				//price_
                nullField = paccessor.paGetPrice();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.BIGDECIMAL.getTypeCode(), null));
                } else {
                   	adapter.setBigDecimal(pstmt, 3, ((java.math.BigDecimal) nullField));
                }
				//itemId_
                nullField = paccessor.paGetItemId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 4, ((java.lang.Integer) nullField));
                }
				//longDesc_
                nullField = paccessor.paGetLongDesc();
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//shortDesc_
                nullField = paccessor.paGetShortDesc();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//mainPage_
                nullField = paccessor.paGetMainPage();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 7, ((java.lang.Boolean) nullField));
                }
				//olstore_type_name_
                nullField = pClassMapping.typeIdPnc.encodeString(paccessor.paGetTypeId(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//itemId_
                nullField = paccessor.paGetItemId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 9, ((java.lang.Integer) nullField));
                }
            } else if (paccessorGen != null) {
            pstmt = connection.prepareStatement(res = "UPDATE olstore_item SET name_ = ?, specialOffer_ = ?, price_ = ?, itemId_ = ?, longDesc_ = ?, shortDesc_ = ?, mainPage_ = ?, olstore_type_name_ = ? WHERE itemId_ = ?");
				//name_
                nullField = paccessorGen.paGetStringField(pClassMapping.name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//specialOffer_
                nullField = paccessorGen.paGetObooleanField(pClassMapping.specialOffer_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 2, ((java.lang.Boolean) nullField));
                }
				//price_
                nullField = paccessorGen.paGetBigDecimalField(pClassMapping.price_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.BIGDECIMAL.getTypeCode(), null));
                } else {
                   	adapter.setBigDecimal(pstmt, 3, ((java.math.BigDecimal) nullField));
                }
				//itemId_
                nullField = paccessorGen.paGetOintField(pClassMapping.itemId_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 4, ((java.lang.Integer) nullField));
                }
				//longDesc_
                nullField = paccessorGen.paGetStringField(pClassMapping.longDesc_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//shortDesc_
                nullField = paccessorGen.paGetStringField(pClassMapping.shortDesc_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//mainPage_
                nullField = paccessorGen.paGetObooleanField(pClassMapping.mainPage_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 7, ((java.lang.Boolean) nullField));
                }
				//olstore_type_name_
                nullField = pClassMapping.typeIdPnc.encodeString(paccessorGen.paGetRefField(pClassMapping.typeId_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//itemId_
                nullField = paccessor.paGetItemId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 9, ((java.lang.Integer) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertolstore_itemTable(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                Olstore_itemAccessor paccessor)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            if (paccessor != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO olstore_item (name_, specialOffer_, price_, itemId_, longDesc_, shortDesc_, mainPage_, olstore_type_name_) VALUES (?, ?, ?, ?, ?, ?, ?, ?)");
				//name_
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//specialOffer_
                nullField = paccessor.paGetSpecialOffer();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 2, ((java.lang.Boolean) nullField));
                }
				//price_
                nullField = paccessor.paGetPrice();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.BIGDECIMAL.getTypeCode(), null));
                } else {
                   	adapter.setBigDecimal(pstmt, 3, ((java.math.BigDecimal) nullField));
                }
				//itemId_
                nullField = paccessor.paGetItemId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 4, ((java.lang.Integer) nullField));
                }
				//longDesc_
                nullField = paccessor.paGetLongDesc();
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//shortDesc_
                nullField = paccessor.paGetShortDesc();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//mainPage_
                nullField = paccessor.paGetMainPage();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 7, ((java.lang.Boolean) nullField));
                }
				//olstore_type_name_
                nullField = pClassMapping.typeIdPnc.encodeString(paccessor.paGetTypeId(conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                pstmt = connection.prepareStatement(res = "INSERT INTO olstore_item (name_, specialOffer_, price_, itemId_, longDesc_, shortDesc_, mainPage_, olstore_type_name_) VALUES (?, ?, ?, ?, ?, ?, ?, ?)");
				//name_
                nullField = paccessorGen.paGetStringField(pClassMapping.name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//specialOffer_
                nullField = paccessorGen.paGetObooleanField(pClassMapping.specialOffer_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 2, ((java.lang.Boolean) nullField));
                }
				//price_
                nullField = paccessorGen.paGetBigDecimalField(pClassMapping.price_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.BIGDECIMAL.getTypeCode(), null));
                } else {
                   	adapter.setBigDecimal(pstmt, 3, ((java.math.BigDecimal) nullField));
                }
				//itemId_
                nullField = paccessorGen.paGetOintField(pClassMapping.itemId_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 4, ((java.lang.Integer) nullField));
                }
				//longDesc_
                nullField = paccessorGen.paGetStringField(pClassMapping.longDesc_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//shortDesc_
                nullField = paccessorGen.paGetStringField(pClassMapping.shortDesc_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//mainPage_
                nullField = paccessorGen.paGetObooleanField(pClassMapping.mainPage_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.OBJBOOLEAN.getTypeCode(), null));
                } else {
                   	adapter.setOboolean(pstmt, 7, ((java.lang.Boolean) nullField));
                }
				//olstore_type_name_
                nullField = pClassMapping.typeIdPnc.encodeString(paccessorGen.paGetRefField(pClassMapping.typeId_NAME, conn));
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteolstore_itemTable(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	Olstore_itemAccessor paccessor) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM olstore_item WHERE itemId_ = ?");
				//itemId_
                nullField = paccessor.paGetItemId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.OBJINT.getTypeCode(), null));
                } else {
                   	adapter.setOint(pstmt, 1, ((java.lang.Integer) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
}

















